@@ -1,4 +1,5 @@
-import urlparse
+from django.utils.encoding import force_text
+from django.utils.six.moves.urllib.parse import parse_qs
 from django.utils.translation import ugettext_lazy as _
 
 from django.db import models
@@ -20,7 +21,7 @@ class ResponseModel(models.Model):
         app_label = 'paypal'
 
     def request(self):
-        request_params = urlparse.parse_qs(self.raw_request)
+        request_params = parse_qs(self.raw_request)
         return self._as_dl(request_params)
     request.allow_tags = True
 
@@ -42,8 +43,8 @@ class ResponseModel(models.Model):
 
     @property
     def context(self):
-        return urlparse.parse_qs(self.raw_response)
+        return parse_qs(self.raw_response)
 
     def value(self, key, default=None):
         ctx = self.context
-        return ctx[key][0].decode('utf8') if key in ctx else default
+        return force_text(ctx[key][0]) if key in ctx else default
