@@ -1,9 +1,12 @@
 import re
 
 from django.db import models
+from django.utils.encoding import python_2_unicode_compatible
+
 from paypal import base
 
 
+@python_2_unicode_compatible
 class ExpressTransaction(base.ResponseModel):
 
     # The PayPal method and version used
@@ -37,7 +40,7 @@ class ExpressTransaction(base.ResponseModel):
     def is_successful(self):
         return self.ack in (self.SUCCESS, self.SUCCESS_WITH_WARNING)
 
-    def __unicode__(self):
-        return u'method: %s: token: %s' % (
+    def __str__(self):
+        return 'method: %s: token: %s' % (
             self.method, self.token)
 
