@@ -1,5 +1,4 @@
 from decimal import Decimal as D
-import urllib
 import logging
 
 from django.views.generic import RedirectView, View
@@ -11,8 +10,11 @@ from django.contrib.auth.models import AnonymousUser
 from django.core.urlresolvers import reverse
 from django.http import HttpResponseRedirect
 from django.db.models import get_model
+from django.utils.http import urlencode
+from django.utils import six
 from django.utils.translation import ugettext_lazy as _
 
+import oscar
 from oscar.apps.checkout.views import PaymentDetailsView, CheckoutSessionMixin
 from oscar.apps.payment.exceptions import UnableToTakePayment
 from oscar.apps.payment.models import SourceType, Source
@@ -22,7 +24,7 @@ from oscar.apps.shipping.methods import FixedPrice, NoShippingRequired
 from paypal.express.facade import get_paypal_url, fetch_transaction_details, confirm_transaction
 from paypal.express.exceptions import (
     EmptyBasketException, MissingShippingAddressException,
-    MissingShippingMethodException)
+    MissingShippingMethodException, InvalidBasket)
 from paypal.exceptions import PayPalError
 
 ShippingAddress = get_model('order', 'ShippingAddress')
@@ -53,20 +55,26 @@ class RedirectView(CheckoutSessionMixin, RedirectView):
             basket = self.request.basket
             url = self._get_redirect_url(basket, **kwargs)
         except PayPalError:
-            messages.error(self.request, _("An error occurred communicating with PayPal"))
+            messages.error(
+                self.request, _("An error occurred communicating with PayPal"))
             if self.as_payment_method:
                 url = reverse('checkout:payment-details')
             else:
                 url = reverse('basket:summary')
             return url
+        except InvalidBasket as e:
+            messages.warning(self.request, six.text_type(e))
+            return reverse('basket:summary')
         except EmptyBasketException:
             messages.error(self.request, _("Your basket is empty"))
             return reverse('basket:summary')
         except MissingShippingAddressException:
-            messages.error(self.request, _("A shipping address must be specified"))
+            messages.error(
+                self.request, _("A shipping address must be specified"))
             return reverse('checkout:shipping-address')
         except MissingShippingMethodException:
-            messages.error(self.request, _("A shipping method must be specified"))
+            messages.error(
+                self.request, _("A shipping method must be specified"))
             return reverse('checkout:shipping-method')
         else:
             # Transaction successfully registered with PayPal.  Now freeze the
@@ -105,7 +113,8 @@ class RedirectView(CheckoutSessionMixin, RedirectView):
                 params['shipping_methods'] = []
 
         else:
-            shipping_methods = Repository().get_shipping_methods(user, basket)
+            shipping_methods = Repository().get_shipping_methods(
+                user=user, basket=basket)
             params['shipping_methods'] = shipping_methods
 
         if settings.DEBUG:
@@ -151,7 +160,9 @@ class SuccessResponseView(PaymentDetailsView):
     preview = True
 
     # We don't have the usual pre-conditions (Oscar 0.7+)
-    pre_conditions = ()
+    @property
+    def pre_conditions(self):
+        return [] if oscar.VERSION[:2] >= (0, 8) else ()
 
     def get(self, request, *args, **kwargs):
         """
@@ -172,12 +183,13 @@ class SuccessResponseView(PaymentDetailsView):
 
         try:
             self.txn = fetch_transaction_details(self.token)
-        except PayPalError, e:
+        except PayPalError as e:
             logger.warning(
                 "Unable to fetch transaction details for token %s: %s",
                 self.token, e)
             messages.error(
-                self.request, _("A problem occurred communicating with PayPal - please try again later"))
+                self.request,
+                _("A problem occurred communicating with PayPal - please try again later"))
             return HttpResponseRedirect(reverse('basket:summary'))
 
         # Reload frozen basket which is specified in the URL
@@ -346,7 +358,6 @@ class SuccessResponseView(PaymentDetailsView):
         # Assume no tax for now
         charge_excl_tax = charge_incl_tax
         method = FixedPrice(charge_excl_tax, charge_incl_tax)
-        method.set_basket(basket)
         name = self.txn.value('SHIPPINGOPTIONNAME')
 
         if not name:
@@ -393,20 +404,27 @@ class ShippingOptionsView(View):
             country=country
         )
         methods = self.get_shipping_methods(user, basket, shipping_address)
-        return self.render_to_response(methods)
+        return self.render_to_response(methods, basket)
 
-    def render_to_response(self, methods):
+    def render_to_response(self, methods, basket):
         pairs = [
             ('METHOD', 'CallbackResponse'),
             ('CURRENCYCODE', self.request.POST.get('CURRENCYCODE', 'GBP')),
         ]
         for index, method in enumerate(methods):
+            if hasattr(method, 'set_basket'):
+                # Oscar < 0.8
+                method.set_basket(basket)
+                charge = method.charge_incl_tax
+            else:
+                cost = method.calculate(basket)
+                charge = cost.incl_tax
+
             pairs.append(('L_SHIPPINGOPTIONNAME%d' % index,
-                          unicode(method.name)))
+                          six.text_type(method.name)))
             pairs.append(('L_SHIPPINGOPTIONLABEL%d' % index,
-                          unicode(method.name)))
-            pairs.append(('L_SHIPPINGOPTIONAMOUNT%d' % index,
-                          method.charge_incl_tax))
+                          six.text_type(method.name)))
+            pairs.append(('L_SHIPPINGOPTIONAMOUNT%d' % index, charge))
             # For now, we assume tax and insurance to be zero
             pairs.append(('L_TAXAMT%d' % index, D('0.00')))
             pairs.append(('L_INSURANCEAMT%d' % index, D('0.00')))
@@ -416,7 +434,7 @@ class ShippingOptionsView(View):
             # No shipping methods available - we flag this up to PayPal indicating that we
             # do not ship to the shipping address.
             pairs.append(('NO_SHIPPING_OPTION_DETAILS', 1))
-        payload = urllib.urlencode(pairs)
+        payload = urlencode(pairs)
         return HttpResponse(payload)
 
     def get_shipping_methods(self, user, basket, shipping_address):
